#include <EEPROM.h>
#include "OneWire.h"
#include "DallasTemperature.h"

#define ONE_WIRE_BUS 5
#define EEPROM_PH4_ADDR 0
#define EEPROM_PH6_86_ADDR 4
#define EEPROM_PH9_18_ADDR 8
#define READ_COUNT 20

#define CALIBRATION_TIME 120000 // 2 นาทีในมิลลิวินาที

int tmpetValue;
int sensorPin = A0; // พินที่ใช้สำหรับเซ็นเซอร์ pH
float ph4Value = 513;

float ph6_86Value = 472;
float ph9_18Value = 407;



unsigned long calibrationStartTime;

// ตัวแปรสำหรับการคำนวณความชันและจุดตัด
float m1, b1, m2, b2;

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

void setup() {
  Serial.begin(9600); // เริ่มต้นการสื่อสารกับ Serial Monitor
  sensors.begin(); // เริ่มต้นเซ็นเซอร์อุณหภูมิ
  
  // อ่านค่าคาลิเบรตจาก EEPROM หรือกำหนดค่าเริ่มต้นถ้าไม่มีค่าใน EEPROM
  EEPROM.get(EEPROM_PH4_ADDR, ph4Value);
  EEPROM.get(EEPROM_PH6_86_ADDR, ph6_86Value);
  EEPROM.get(EEPROM_PH9_18_ADDR, ph9_18Value);

  if (ph4Value == 0xFFFF || ph4Value <= 0) {
    ph4Value = 513;
    EEPROM.put(EEPROM_PH4_ADDR, ph4Value);
  }

  if (ph6_86Value == 0xFFFF || ph6_86Value <= 0) {
    ph6_86Value = 472;
    EEPROM.put(EEPROM_PH6_86_ADDR, ph6_86Value);
  }

  if (ph9_18Value == 0xFFFF || ph9_18Value <= 0) {
    ph9_18Value = 407;
    EEPROM.put(EEPROM_PH9_18_ADDR, ph9_18Value);
  }

  // คำนวณค่าความชันและจุดตัด
  calculateSlopes();

  Serial.print("ค่าคาลิเบรตที่อ่านได้: ");
  Serial.print("pH 4.0 = ");
  Serial.print(ph4Value);
  Serial.print(", pH 6.86 = ");
  Serial.print(ph6_86Value);
  Serial.print(", pH 9.18 = ");
  Serial.println(ph9_18Value);

  Serial.println("ป้อน 'cal' เพื่อเริ่มการคาลิเบรต");
}

void loop() {
  if (Serial.available() > 0) {
    String command = Serial.readStringUntil('\n');
    command.trim();

    if (command.equalsIgnoreCase("cal")) {
      Serial.println("กรุณาใส่เซนเซอร์ pH ลงในสารละลาย pH 4.0 และกด ENTER");
      waitForEnter();
      delay_loop();
      ph4Value = medianOfReadings();
      EEPROM.put(EEPROM_PH4_ADDR, ph4Value);
      Serial.println("ค่าที่บันทึกสำหรับ pH 4.0: " + String(ph4Value));
      

      Serial.println("กรุณาใส่เซนเซอร์ pH ลงในสารละลาย pH 6.86 และกด ENTER");
      waitForEnter();
      delay_loop();
      ph6_86Value = medianOfReadings();
      EEPROM.put(EEPROM_PH6_86_ADDR, ph6_86Value);
      Serial.println("ค่าที่บันทึกสำหรับ pH 6.86: " + String(ph6_86Value));


      Serial.println("กรุณาใส่เซนเซอร์ pH ลงในสารละลาย pH 9.18 และกด ENTER");
      waitForEnter();
      delay_loop();
      ph9_18Value = medianOfReadings();
      EEPROM.put(EEPROM_PH9_18_ADDR, ph9_18Value);
      Serial.println("ค่าที่บันทึกสำหรับ pH 9.18: " + String(ph9_18Value));


      // คำนวณค่าความชันและจุดตัดใหม่
      calculateSlopes();
      Serial.println("การคาลิเบรตเสร็จสิ้น");
    }
  }

  sensors.requestTemperatures(); // ขอค่าอุณหภูมิจากเซ็นเซอร์
  tmpetValue = sensors.getTempCByIndex(0); // รับค่าความร้อนในองศาเซลเซียส

  int analogValue = medianOfReadings(); // อ่านค่าจากเซ็นเซอร์ pH และใช้ค่ากลาง
  float phValue = getPHvalue(analogValue); // คำนวณค่า pH

  // แสดงค่า analog และค่า pH ที่คำนวณได้
  Serial.print("ค่า Analog: ");
  Serial.print(analogValue);
  Serial.print("  ค่า อุหภูมิ: ");
  Serial.print(tmpetValue);
  Serial.print("  ค่า pH: ");
  Serial.println(phValue);


  delay(1000); // หน่วงเวลา 1 วินาที
}

// ฟังก์ชันรอการกดปุ่ม ENTER
void waitForEnter() {
  while (Serial.available() == 0) {
    // รอจนกว่าจะมีข้อมูลจาก Serial Monitor
  }
  Serial.read(); // อ่านข้อมูลที่มีอยู่ใน Serial Monitor
}

// ฟังก์ชันคำนวณค่าความชันและจุดตัด
void calculateSlopes() {
  // คำนวณค่าความชันและจุดตัดสำหรับแต่ละช่วง
  // ช่วง pH 4.0 - 6.86
  m1 = (6.86 - 4.0) / (ph6_86Value - ph4Value);
  b1 = 4.0 - m1 * ph4Value;

  // ช่วง pH 6.86 - 9.18
  m2 = (9.18 - 6.86) / (ph9_18Value - ph6_86Value);
  b2 = 6.86 - m2 * ph6_86Value;
}

// ฟังก์ชันการคำนวณค่า pH
float getPHvalue(int analogValue) {
  float PH_VALUE = 0;

  // การคำนวณค่า pH จากจุดการคาลิเบรต
  if (analogValue >= ph6_86Value && analogValue <= 1024) {
    PH_VALUE = m1 * analogValue + b1; // ช่วง pH 4.0 - 6.86

  } else if (analogValue >= 0  && analogValue < ph6_86Value) {
    PH_VALUE = m2 * analogValue + b2; // ช่วง pH 6.86 - 9.18
    

  } else {
    PH_VALUE = -1; // ค่า pH นอกช่วงการคาลิเบรต
  }

  // ตรวจสอบให้ค่า pH อยู่ในช่วงที่ถูกต้อง
  if (PH_VALUE > 14.0) PH_VALUE = 14.0;
  if (PH_VALUE < 0.0) PH_VALUE = 0.0;

  return PH_VALUE;
}

// ฟังก์ชันเพื่ออ่านค่าจากเซ็นเซอร์ pH หลายๆ ค่าและหาค่ากลาง
float medianOfReadings() {
  #define ARRY_LENGTH 10
  float PH_VALUE = 0;
  int pv[ARRY_LENGTH];
  for(int i = 0; i < ARRY_LENGTH; i++ ){
    pv[i] = analogRead(sensorPin);
    delay(1);
  }
  for(int i = 0; i < ARRY_LENGTH; i++){
    for(int k = i; k < ARRY_LENGTH; k++ ){
      if( pv[i] < pv[k] ){
        int tmp = pv[i];
        pv[i] = pv[k];
        pv[k] = tmp;
      }
    }
  }

  //เริ่มการชดเชยอุณหภูมิ
  if( tmpetValue > 42 )  pv[ARRY_LENGTH/2] += 5;
  else if(tmpetValue > 28){
    pv[ARRY_LENGTH/2] += 5*(tmpetValue - 28)/14;
  }
  //สิ้นสุดการชดเชยอุณหภูมิ
  
  return PH_VALUE = pv[ARRY_LENGTH/2];
}


void delay_loop(){
   
    for(int i = 120; i > 0; i--){
      int analogValue = medianOfReadings();
      Serial.print("กำลัง Cal(");
      Serial.print(analogValue);
      Serial.print(") ใช้เวลาอีก : ");
      Serial.print(i);
      Serial.println(" วินาที");
      delay(1000); // อัพเดททุกวินาที
      }
}